clear all;
close all;
clc;


% Parameters
eta = 2;                % Pareto shape parameter of productivity distribution
al = 0.3;               % capital share in production
r = 0.025;              % world intedst rate
d = 0.2;                % impatience of phi entdpdneurs
la = 1.5;               % financial friction
A = 1;                  % productivity
phi = 1/2;

%AUXILIARY PARAMETERS
ga = (al/eta)/(al/eta +(1-al));
Theta = (r/al)*(eta*la/(eta-1)*(al*A/r)^(eta/al))^ga;

tauss = ga/((1-ga)*(d/r) + ga);
xss = (al/eta*Theta/(d-r))^((phi+ga)/(phi*(1-ga)))*((1-tauss)*(1-al)*Theta)^(1/phi);

I=200;
tau1max = 1;
tau1min = -1;
tau1 = linspace(tau1min,tau1max,I)';
x_locus = (al/eta*Theta/(d-r))^((phi+ga)/(phi*(1-ga))).*((1-tau1)*(1-al)*Theta).^(1/phi);

J= 200;
tau2max = ga;
tau2min = -1;
tau2 = linspace(tau2min,tau2max,J)';
tau_locus = (ga*(1-tau2)*al/eta*Theta./(d*(ga-tau2))).^((phi+ga)/(phi*(1-ga))).*((1-tau2)*(1-al)*Theta).^(1/phi);

plot(x_locus,tau1,tau_locus,tau2,xss,tauss,'o')
xlim([0.5*xss 1.5*xss])
ylim([-.2 .2])

N = 4000;
dt = 0.01;
time = (1:N)*dt;
weight = 0;
x0 = 1.5*xss;
tau0 = .0530054;

x_t(1) = x0;
tau_t(1) = tau0;

for n=1:N-1
    l_t(n) = ((1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(1-tau_t(n))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (r-d)*x_t(n)) + x_t(n);
end

x20 = .3*xss;
tau20 = -0.084542;

x2_t(1) = x20;
tau2_t(1) = tau20;

for n=1:N-1
    l2_t(n) = ((1-tau2_t(n))*(1-al)*Theta*x2_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(1-tau2_t(n))*al/eta*Theta*l2_t(n)^(1-ga)*x2_t(n)^(ga-1);
    tau2_t(n+1) = dt*(d*(tau2_t(n)-ga) + xxx) + tau2_t(n);
    x2_t(n+1)  = dt*(al/eta*Theta*x2_t(n)^ga.*l2_t(n)^(1-ga) + (r-d)*x2_t(n)) + x2_t(n);
end


x_locus = x_locus/xss;
tau_locus = tau_locus/xss;
x_t = x_t/xss;
x2_t = x2_t/xss;

%PHASE DIAGRAM
figure(1)
h1 = plot(x_locus,zeros(1,I),'k',x_locus,tau1,'--b',tau_locus,tau2,'--b',1,tauss,'o',x_t,tau_t,'r',x2_t,tau2_t,'r','LineWidth',2)
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
% set(gca, 'YTick', []);
% set(gca, 'XTick', []);
text(0.71,-.045,'$\dot{\tau}_\ell=0$','FontSize',16,'interpreter','latex')
text(0.78,0.07,'$\dot{x}=0$','FontSize',16,'interpreter','latex')
text(0.48,0.021,'Optimal Trajectory','FontSize',16,'interpreter','latex')
xlim([0.4 1.5])
ylim([-.05 .08])
ylabel('$\tau_\ell$','FontSize',16,'interpreter','latex')
xlabel('$x$','FontSize',16,'interpreter','latex')
lengtharrowy = 0.02;
lengtharrowx = 0.02*(1.5-0.4)/(0.08+.05)/1.33;
arrow([.45 -0.042],[.45 -0.042+lengtharrowy]); arrow([.45 -0.042],[.45+lengtharrowx -0.042]);
arrow([1.48 0.052],[1.48 0.052-lengtharrowy]); arrow([1.48 0.052],[1.48-lengtharrowx 0.052]);
arrow([1.12 0.05],[1.12 0.05+lengtharrowy]); arrow([1.12 0.05],[1.12-lengtharrowx 0.05]);
arrow([0.9 -0.02],[0.9 -0.02-lengtharrowy]); arrow([0.9 -0.02],[0.9+lengtharrowx -0.02]);

print -depsc graphs/fig1.eps