clear all; clc; close all;

set(groot,'DefaultAxesColorOrder',[0 0 1; 0 .5 0; 1 0 0; 0 .75 .75; ...
                                   .75 0 .75; .75 .75 0; .25 .25 .25])


% Parameters
eta = 1.06;                % Pareto shape parameter of productivity distribution
al = 1/3;               % capital share in production
rho = 0.03;               % world interest rate
d = 0.1;                % impatience of entrepreneurs
la = 2;                % financial friction
A = 1;                  %productivity
%PREFERENCE PARAMETERS
sigma = 1;
Frisch = 1;
phi = 1/Frisch; a0=1;

%COMMENT: If we can somehow defend d=0.13 and Frisch=0.3, the figures look
%even nicer

%AUXILIARY PARAMETERS
ga = (al/eta)/(al/eta +(1-al));
Theta = (rho/al)*(eta*la/(eta-1)*(al*A/rho)^(eta/al))^ga;

tau_bar = ga/(ga + (1-ga)*(d/rho));

b0 = 0; %INITIAL BOND HOLDINGS OF WORKERS

%FIND mu_bar that solves system
mu_min_0 = .0001;
mu_max_0 = 1;
mu_bar0 = .6;

maxit = 30;
crit = 10^(-5);

T = 150;
N = 10000;
% T = 200;
% N = 15000;
dt = T/N;
time = dt*(1:N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPETITIVE EQUILIBRIUM %
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIND mu_bar that solves system
mu_bar_CE = mu_bar0;
mu_min_CE = mu_min_0;
mu_max_CE = mu_max_0;

for i=1:maxit

%SEPARABLE UTILITY
x_bar_CE = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar_CE/a0*(1-al)*Theta)^(1/phi);
l_bar_CE = (mu_bar_CE/a0*(1-al)*Theta*x_bar_CE^ga).^(1/(phi+ga));
c_bar_CE = mu_bar_CE^(-1/sigma); %SEPARABLE

x0 = .1*x_bar_CE; %INITIAL WEALTH OF ENTREPRENEURS

XX_CE(1) = x0;
for n=1:N
    LL_CE(n) = (mu_bar_CE/a0.*(1-al)*Theta*XX_CE(n)^ga).^(1/(phi+ga));
    dXX_CE(n) = al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n);
    XX_CE(n+1)  = dt*(al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n)) + XX_CE(n);
end
XX_CE(N+1)=[];

YY_CE = Theta*XX_CE.^ga.*LL_CE.^(1-ga);
yPDV_CE = sum(dt*YY_CE.*exp(-rho.*time)) + exp(-rho*T)*YY_CE(N)/rho;

F_CE(i) = b0 + (1-al)*yPDV_CE - c_bar_CE/rho;
mu_i_CE(i)=mu_bar_CE;

%UPDATE INTEREST RATE
if F_CE(i)>crit
    disp('F>0')
    mu_max_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_min_CE);
elseif F_CE(i)<-crit;
    disp('F<0')
    mu_min_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_max_CE);
elseif abs(F_CE(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_CE)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort_CE index_CE] = sort(mu_i_CE);
F_sort_CE = F_CE(index_CE);
%close all
plot(mu_sort_CE,F_sort_CE,mu_sort_CE,zeros(1,length(F_CE)),'--')

CC_CE = c_bar_CE*ones(1,N);
if sigma == 1 
    VV_CE = log(CC_CE) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = log(c_bar_CE) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
else
    VV_CE = CC_CE.^(1-sigma)./(1-sigma) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = c_bar_CE.^(1-sigma)./(1-sigma) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
end
WW_CE = (1-al)*YY_CE./LL_CE;
KK_CE = al*(eta-1)/eta*YY_CE./rho;
ZZ_CE = YY_CE./(KK_CE.^al.*LL_CE.^(1-al));
y_bar_CE = Theta*x_bar_CE.^ga.*l_bar_CE.^(1-ga);
w_bar_CE = (1-al)*y_bar_CE./l_bar_CE;
k_bar_CE = al*(eta-1)/eta*y_bar_CE./rho;
Z_bar_CE = y_bar_CE./(k_bar_CE.^al.*l_bar_CE.^(1-al));


%%%%%%%%%%%%%%%%%%%%%
% PLANNER'S PROBLEM %
%%%%%%%%%%%%%%%%%%%%%

mu_bar = mu_bar0;
mu_min = mu_min_0;
mu_max = mu_max_0;

maxit_tau = 40;
crit = 10^(-6);
% T1 = 120;
T1 = 70;
N1 = round(T1/dt);
% maxit = 1;
% mu_bar = .2;
for i=1:maxit

%SEPARABLE UTILITY
x_bar = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar/a0*(1-tau_bar)*(1-al)*Theta)^(1/phi);
l_bar = (mu_bar/a0*(1-tau_bar)*(1-al)*Theta*x_bar^ga).^(1/(phi+ga));
c_bar = mu_bar^(-1/sigma); %SEPARABLE

%x0 = .1*x_bar; %INITIAL WEALTH OF ENTREPRENEURS
%x0 = x_bar; %INITIAL WEALTH OF ENTREPRENEURS

tau0 = -0.17;
tau_max = tau_bar;
% tau_min = -0.5;
tau_min = -0.6;

%SHOOTING ALGORITHM FINDING tau0
for j=1:maxit_tau
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N-1
    l_t(n) = (mu_bar/a0.*(max(1-tau_t(n),0))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(max(1-tau_t(n),0))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
    x_t(n+1) = max(x_t(n+1),0.000001);
end


tau_LR(j) = tau_t(N1)-tau_t(N1-1);
tau_min_j(j) = tau_min;
tau_max_j(j) = tau_max;
tau0_j(j) = tau0;

tau_t_save(:,j) = tau_t;

%UPDATE tau0
if tau_LR(j)>crit
    %disp('tau_LR>0')
    tau_max = tau0;
    tau0 = 0.5*(tau0+tau_min);
elseif tau_LR(j)<-crit;
    %disp('tau_LR<0')
    tau_min = tau0;
    tau0 = 0.5*(tau0+tau_max);
elseif abs(tau_LR(j))<crit;
    display('Shooting algorithm converged, tau0 =')
    disp(tau0)
    break
end

if j==maxit_tau
    disp('Shooting algorithm did not converge fully, tau0 = ')
    disp(tau0)
    %If algorithm doesn't converge, take the tau0 that works the best out
    %of all tau0s
    [obj index] = min(abs(tau_LR(1:j)));
    tau0 = tau0_j(index);
end

end

%plot([tau_min_j',tau0_j',tau_max_j'])

%RUN SYSTEM ONE MORE TIME TO GET NICE TIME PATHS FOR tau and x.
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N
    l_t(n) = (mu_bar/a0.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*al/eta*(1-tau_t(n))*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    if n>N1
        tau_t(n+1) = tau_t(n);
    end
    dXX(n) = al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end
x_t(N+1)=[];
tau_t(N+1)=[];

YY = Theta*x_t.^ga.*l_t.^(1-ga);
yPDV = sum(dt*YY.*exp(-rho.*time)) + exp(-rho*T)*YY(N)/rho;

F(i) = b0 + (1-al)*yPDV - c_bar/rho;
mu_i(i)=mu_bar;

%UPDATE INTEREST RATE
if F(i)>crit
    disp('F>0')
    mu_max = mu_bar;
    mu_bar = 0.5*(mu_bar+mu_min);
elseif F(i)<-crit;
    disp('F<0')
    mu_min = mu_bar;
    mu_bar = 0.5*(mu_bar+mu_max);
elseif abs(F(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort index] = sort(mu_i);
F_sort = F(index);
plot(mu_sort,F_sort,mu_sort,zeros(1,length(F)),'--')

%COMPUTE AND PLOT SOME VARIABLES AND COMPARE TO COMPETITIVE EQUILIBRIUM
LL = l_t;
XX = x_t;
TAU = tau_t;
CC = c_bar*ones(1,N);
if sigma == 1 
    VV = log(CC) - a0*LL.^(1+phi)./(1+phi); %Welfare
else
    VV = CC.^(1-sigma)./(1-sigma) - a0*LL.^(1+phi)./(1+phi); %Welfare
end
WW = (1-al)*YY./LL;
KK = al*(eta-1)/eta*YY./rho;
ZZ = YY./(KK.^al.*LL.^(1-al));
y_bar = Theta*x_bar.^ga.*l_bar.^(1-ga);
w_bar = (1-al)*y_bar./l_bar;
k_bar = al*(eta-1)/eta*y_bar./rho;
Z_bar = y_bar./(k_bar.^al.*l_bar.^(1-al));




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTING SOME NUMBERS OF INTEREST %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Largest difference in entrepreneurial wealth = ')
disp(max((XX - XX_CE)./XX_CE))
disp('Largest difference in Labor Supply = ')
disp(max((LL - LL_CE)./LL_CE))
disp('Largest difference in Wage = ')
disp(min((WW - WW_CE)./WW_CE))
disp('Largest difference in GDP = ')
disp(max((YY - YY_CE)./YY_CE))

%TIME ZERO WELFARE
%WELF0 = V_bar/rho;
WELF0 = sum(dt*VV.*exp(-rho.*time)) + exp(-rho*T)*VV(N)/rho;
WELF0_CE = sum(dt*VV_CE.*exp(-rho.*time)) + exp(-rho*T)*VV_CE(N)/rho;
if sigma ~=1
CWELF0 = sum(dt*CC.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC(N).^(1-sigma)/(1-sigma)/rho;
CWELF0_CE = sum(dt*CC_CE.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC_CE(N).^(1-sigma)/(1-sigma)/rho;
LWELF0 = sum(dt*a0*LL.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL(N).^(1+phi)/(1+phi)/rho;
LWELF0_CE = sum(dt*a0*LL_CE.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL_CE(N).^(1+phi)/(1+phi)/rho;
end

%PERIOD BY PERIOD CONSUMPTION EQUIVALENT
VV_diff = VV - VV_CE;
CEQ_t = exp(VV_diff) -1;

%TIME ZERO CONSUMPTION EQUIVALENT
WELF0_diff = WELF0 - WELF0_CE;
CEQ0 = exp(rho*WELF0_diff) -1;

if sigma ~=1
   CEQ0 = ((CWELF0 - LWELF0 + LWELF0_CE)/CWELF0_CE).^(1/(1-sigma))-1;
end

disp('Welfare Gain (Consumption Equivalents) = ')
disp(CEQ0)

%%%%%%%%%%%%%%%%%%%%
% FIGURE FOR PAPER %
%%%%%%%%%%%%%%%%%%%%
Tmax = 80;
figure(1)
set(gcf,'PaperPosition',[0 0 15 5])
subplot(1,2,1)
h1 = plot(time,zeros(1,N),'--',time,TAU,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(a) Labor Tax, $\tau_\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(1,2,2)
h1 = plot(time,XX_CE/x_bar_CE,'--',time,XX/x_bar_CE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')
print -depsc graphs/fig2_levels.eps

figure(2)
set(gcf,'PaperPosition',[0 0 15 18])
subplot(3,2,1)
h1 = plot(time,zeros(1,N),'--',time,(LL-LL_CE)./LL_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(a) Labor Supply, $\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,2)
h1 = plot(time,zeros(1,N),'--',time,(XX-XX_CE)./XX_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,3)
h1 = plot(time,zeros(1,N),'--',time,(WW-WW_CE)./WW_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(c) Wage, $w$, and Labor Productivity, $y/\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,4)
h1 = plot(time,zeros(1,N),'--',time,(ZZ-ZZ_CE)./ZZ_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(d) Total Factor Productivity, $Z$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,5)
h1 = plot(time,zeros(1,N),'--',time,(YY-YY_CE)./YY_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(e) Income, $y$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,6)
h1 = plot(time,zeros(1,N),'--',time,CEQ_t,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(f) Worker Period Utility, $u(c,\ell)$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

print -depsc graphs/fig2_deviations.eps


%saving variables for hand-to-mouth figure
time_bench = time;
TAU_bench = TAU; XX_CE_bench = XX_CE; XX_bench = XX; x_bar_CE_bench = x_bar_CE;
LL_bench = LL; LL_CE_bench = LL_CE;
WW_bench = WW; WW_CE_bench = WW_CE;
ZZ_bench = ZZ; ZZ_CE_bench = ZZ_CE;
YY_bench = YY; YY_CE_bench = YY_CE;
CEQ_t_bench = CEQ_t;
save data/fig2_3_save.mat time_bench TAU_bench XX_CE_bench  XX_bench x_bar_CE_bench LL_bench LL_CE_bench WW_bench WW_CE_bench ZZ_bench ZZ_CE_bench YY_bench YY_CE_bench CEQ_t_bench

%close all

figure(3)
set(gcf,'PaperPosition',[0 0 20 12])
subplot(2,3,1)
h1 = plot(time,zeros(1,N),'--',time,(LL-LL_CE)./LL_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(a) Labor Supply, $\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(2,3,2)
h1 = plot(time,zeros(1,N),'--',time,(XX-XX_CE)./XX_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(2,3,3)
h1 = plot(time,zeros(1,N),'--',time,(WW-WW_CE)./WW_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(c) Wage, $w$, and Labor Productivity, $y/\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(2,3,4)
h1 = plot(time,zeros(1,N),'--',time,(ZZ-ZZ_CE)./ZZ_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(d) Total Factor Productivity, $Z$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(2,3,5)
h1 = plot(time,zeros(1,N),'--',time,(YY-YY_CE)./YY_CE,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(e) Income, $y$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(2,3,6)
h1 = plot(time,zeros(1,N),'--',time,CEQ_t,'LineWidth',2);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(f) Worker Period Utility, $u(c,\ell)$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')


print -depsc graphs/extras/fig2_deviations_slides.eps