clear all; clc; close all;

set(groot,'DefaultAxesColorOrder',[0 0 1; 0 .5 0; 1 0 0; 0 .75 .75; ...
                                   .75 0 .75; .75 .75 0; .25 .25 .25])


% Parameters
eta = 1.06;                % Pareto shape parameter of productivity distribution
al = 1/3;               % capital share in production
rho = 0.03;               % world interest rate
d = 0.1;                % impatience of entrepreneurs
la = 2;                % financial friction
A = 1;                  %productivity
%PREFERENCE PARAMETERS
sigma = 1;
Frisch = 1;
phi = 1/Frisch; a0=1;

Smax = 0.01;

%COMMENT: If we can somehow defend d=0.13 and Frisch=0.3, the figures look
%even nicer

%AUXILIARY PARAMETERS
ga = (al/eta)/(al/eta +(1-al));
Theta = (rho/al)*(eta*la/(eta-1)*(al*A/rho)^(eta/al))^ga;

tau_bar = ga/(ga + (1-ga)*(d/rho));

b0 = 0; %INITIAL BOND HOLDINGS OF WORKERS

%FIND mu_bar that solves system
mu_min_0 = .0001;
mu_max_0 = 1;
mu_bar0 = .6;

maxit = 30;
crit = 10^(-5);

T = 150;
N = 10000;
% T = 200;
% N = 15000;
dt = T/N;
time = dt*(1:N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPETITIVE EQUILIBRIUM %
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIND mu_bar that solves system
mu_bar_CE = mu_bar0;
mu_min_CE = mu_min_0;
mu_max_CE = mu_max_0;

for i=1:maxit

%SEPARABLE UTILITY
x_bar_CE = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar_CE/a0*(1-al)*Theta)^(1/phi);
l_bar_CE = (mu_bar_CE/a0*(1-al)*Theta*x_bar_CE^ga).^(1/(phi+ga));
c_bar_CE = mu_bar_CE^(-1/sigma); %SEPARABLE

x0 = .1*x_bar_CE; %INITIAL WEALTH OF ENTREPRENEURS

XX_CE(1) = x0;
for n=1:N
    LL_CE(n) = (mu_bar_CE/a0.*(1-al)*Theta*XX_CE(n)^ga).^(1/(phi+ga));
    dXX_CE(n) = al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n);
    XX_CE(n+1)  = dt*(al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n)) + XX_CE(n);
end
XX_CE(N+1)=[];

YY_CE = Theta*XX_CE.^ga.*LL_CE.^(1-ga);
yPDV_CE = sum(dt*YY_CE.*exp(-rho.*time)) + exp(-rho*T)*YY_CE(N)/rho;

F_CE(i) = b0 + (1-al)*yPDV_CE - c_bar_CE/rho;
mu_i_CE(i)=mu_bar_CE;

%UPDATE INTEREST RATE
if F_CE(i)>crit
    disp('F>0')
    mu_max_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_min_CE);
elseif F_CE(i)<-crit;
    disp('F<0')
    mu_min_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_max_CE);
elseif abs(F_CE(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_CE)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort_CE index_CE] = sort(mu_i_CE);
F_sort_CE = F_CE(index_CE);
close all
% plot(mu_sort_CE,F_sort_CE,mu_sort_CE,zeros(1,length(F_CE)),'--')

CC_CE = c_bar_CE*ones(1,N);
if sigma == 1 
    VV_CE = log(CC_CE) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = log(c_bar_CE) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
else
    VV_CE = CC_CE.^(1-sigma)./(1-sigma) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = c_bar_CE.^(1-sigma)./(1-sigma) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
end
WW_CE = (1-al)*YY_CE./LL_CE;
KK_CE = al*(eta-1)/eta*YY_CE./rho;
ZZ_CE = YY_CE./(KK_CE.^al.*LL_CE.^(1-al));
y_bar_CE = Theta*x_bar_CE.^ga.*l_bar_CE.^(1-ga);
w_bar_CE = (1-al)*y_bar_CE./l_bar_CE;
k_bar_CE = al*(eta-1)/eta*y_bar_CE./rho;
Z_bar_CE = y_bar_CE./(k_bar_CE.^al.*l_bar_CE.^(1-al));





%%%%%%%%%%%%%%%%%%%%%%%
% UNLIMITED TRANSFERS %
%%%%%%%%%%%%%%%%%%%%%%%
mu_bar_UL = mu_bar0;
mu_min_UL = mu_min_0;
mu_max_UL = mu_max_0;

for i=1:maxit
x_bar_UL = (al/eta*Theta/d)^((phi+ga)/(phi*(1-ga)))*(mu_bar_UL*(1-al)*Theta)^(1/phi);
l_bar_UL = (mu_bar_UL*(1-al)*Theta*x_bar_UL^ga).^(1/(phi+ga));
y_bar_UL = Theta*x_bar_UL^ga*l_bar_UL^(1-ga);
c_bar_UL = mu_bar_UL^(-1/sigma);

F_UL(i) = b0 + (1-al)*y_bar_UL/rho - c_bar_UL/rho;
mu_i_UL(i)=mu_bar_UL;

%UPDATE INTEREST RATE
if F_UL(i)>crit
    disp('F>0')
    mu_max_UL = mu_bar_UL;
    mu_bar_UL = 0.5*(mu_bar_UL+mu_min_UL);
elseif F_UL(i)<-crit;
    disp('F<0')
    mu_min_UL = mu_bar_UL;
    mu_bar_UL = 0.5*(mu_bar_UL+mu_max_UL);
elseif abs(F_UL(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_UL)
    break
end
end

s_x_bar = -rho;
S_bar = s_x_bar*x_bar_UL;
w_bar_UL = (1-al)*y_bar_UL/l_bar_UL;

XX_UL(1) = x0;
XX_UL(2:N)= x_bar_UL;
SIGMA_UL(1) = .04;
SIGMA_UL(2:N)= s_x_bar;
LL_UL(1) = LL_CE(1);
LL_UL(2:N) = l_bar_UL;
WW_UL(1) = WW_CE(1);
WW_UL(2:N) = w_bar_UL;

CC_UL(1:N) = c_bar_UL;

VV_UL = log(CC_UL) - a0*LL_UL.^(1+phi)./(1+phi); %Welfare

%TIME ZERO WELFARE
WELF0_UL = sum(dt*VV_UL.*exp(-rho.*time)) + exp(-rho*T)*VV_UL(N)/rho;
WELF0_CE = sum(dt*VV_CE.*exp(-rho.*time)) + exp(-rho*T)*VV_CE(N)/rho;

%PERIOD BY PERIOD CONSUMPTION EQUIVALENT
VV_diff = VV_UL - VV_CE;
CEQ_t = exp(VV_diff) -1;

%TIME ZERO CONSUMPTION EQUIVALENT
WELF0_diff = WELF0_UL - WELF0_CE;
CEQ0 = exp(rho*WELF0_diff) -1;

disp('Welfare Gain (Consumption Equivalents) = ')
disp(CEQ0)

%%%%%%%%%%%%%%%%%%%%
% FIGURE FOR PAPER %
%%%%%%%%%%%%%%%%%%%%
% figure(2)
% set(gcf,'PaperPosition',[0 0 15 10])
% subplot(2,2,1)
% h1 = plot(time,zeros(1,N),'--',time,SIGMA_UL,'LineWidth',2);
% set(gca,'FontSize',16)
% legend(h1,'Equilibrium','Planner','Location','NorthEast')
% xlim([-2 T])
% ylim([-0.035 0.035])
% title('(a) Transfer','interpreter','latex')
% ylabel('Transfer, $\varsigma_x x$','interpreter','latex')
% xlabel('Years','FontSize',16,'interpreter','latex')
% 
% subplot(2,2,2)
% h1 = plot(time,LL_CE/l_bar_CE,'--',time,LL_UL/l_bar_CE,'LineWidth',2);
% set(gca,'FontSize',16)
% legend(h1,'Equilibrium','Planner','Location','SouthEast')
% xlim([-2 T])
% title('(b) Labor Supply','interpreter','latex')
% ylabel('Labor Supply, $\ell$','interpreter','latex')
% xlabel('Years','FontSize',16,'interpreter','latex')
% 
% subplot(2,2,3)
% h1 = plot(time,WW_CE/w_bar_CE,'--',time,WW_UL/w_bar_CE,'LineWidth',2);
% set(gca,'FontSize',16)
% legend(h1,'Equilibrium','Planner','Location','SouthEast')
% xlim([-2 T])
% title('(c) Wage/Labor Productivity','interpreter','latex')
% ylabel('Wage $w$/Labor Productivity $y/\ell$','FontSize',16,'interpreter','latex')
% xlabel('Years','FontSize',16,'interpreter','latex')
% 
% subplot(2,2,4)
% h1 = plot(time,XX_CE/x_bar_CE,'--',time,XX_UL/x_bar_CE,'LineWidth',2);
% set(gca,'FontSize',16)
% legend(h1,'Equilibrium','Planner','Location','SouthEast')
% xlim([-2 T])
% title('(d) Entrepreneurial Wealth','interpreter','latex')
% ylabel('Entrepreneurial Wealth','FontSize',16,'interpreter','latex')
% xlabel('Years','FontSize',16,'interpreter','latex')

Tmax = 80;

figure(2)
set(gcf,'PaperPosition',[0 0 15 5])
subplot(1,2,1)
h1 = plot(time,zeros(1,N),'--',time,SIGMA_UL,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','NorthEast')
xlim([-1 Tmax])
ylim([-0.035 0.035])
title('(a) Transfer, $\varsigma_x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(1,2,2)
h1 = plot(time,XX_CE/x_bar_CE,'--',time,XX_UL/x_bar_CE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([-1 Tmax])
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

print -depsc graphs/appendix/fig4.eps


