clear all; clc; close all;

set(groot,'DefaultAxesColorOrder',[0 0 1; 0 .5 0; 1 0 0; 0 .75 .75; ...
                                   .75 0 .75; .75 .75 0; .25 .25 .25])


% Parameters
eta = 1.06;                % Pareto shape parameter of productivity distribution
al = 1/3;               % capital share in production
rho = 0.03;               % world interest rate
d = 0.1;                % impatience of entrepreneurs
la = 2;                % financial friction
A = 1;                  %productivity
%PREFERENCE PARAMETERS
sigma = 1;
Frisch = 1;
phi = 1/Frisch; a0=1;

Smax = .5;

%COMMENT: If we can somehow defend d=0.13 and Frisch=0.3, the figures look
%even nicer

%AUXILIARY PARAMETERS
ga = (al/eta)/(al/eta +(1-al));
Theta = (rho/al)*(eta*la/(eta-1)*(al*A/rho)^(eta/al))^ga;

tau_bar = ga/(ga + (1-ga)*(d/rho));

b0 = 0; %INITIAL BOND HOLDINGS OF WORKERS

%FIND mu_bar that solves system
mu_min_0 = .0001;
mu_max_0 = 1;
mu_bar0 = .6;

maxit = 30;
crit = 10^(-5);

T = 150;
N = 10000;
% T = 200;
% N = 15000;
dt = T/N;
time = dt*(1:N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPETITIVE EQUILIBRIUM %
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIND mu_bar that solves system
mu_bar_CE = mu_bar0;
mu_min_CE = mu_min_0;
mu_max_CE = mu_max_0;

for i=1:maxit

%SEPARABLE UTILITY
x_bar_CE = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar_CE/a0*(1-al)*Theta)^(1/phi);
l_bar_CE = (mu_bar_CE/a0*(1-al)*Theta*x_bar_CE^ga).^(1/(phi+ga));
c_bar_CE = mu_bar_CE^(-1/sigma); %SEPARABLE

x0 = .1*x_bar_CE; %INITIAL WEALTH OF ENTREPRENEURS

XX_CE(1) = x0;
for n=1:N
    LL_CE(n) = (mu_bar_CE/a0.*(1-al)*Theta*XX_CE(n)^ga).^(1/(phi+ga));
    dXX_CE(n) = al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n);
    XX_CE(n+1)  = dt*(al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n)) + XX_CE(n);
end
XX_CE(N+1)=[];

YY_CE = Theta*XX_CE.^ga.*LL_CE.^(1-ga);
yPDV_CE = sum(dt*YY_CE.*exp(-rho.*time)) + exp(-rho*T)*YY_CE(N)/rho;

F_CE(i) = b0 + (1-al)*yPDV_CE - c_bar_CE/rho;
mu_i_CE(i)=mu_bar_CE;

%UPDATE INTEREST RATE
if F_CE(i)>crit
    disp('F>0')
    mu_max_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_min_CE);
elseif F_CE(i)<-crit;
    disp('F<0')
    mu_min_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_max_CE);
elseif abs(F_CE(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_CE)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort_CE index_CE] = sort(mu_i_CE);
F_sort_CE = F_CE(index_CE);
close all
plot(mu_sort_CE,F_sort_CE,mu_sort_CE,zeros(1,length(F_CE)),'--')

CC_CE = c_bar_CE*ones(1,N);
if sigma == 1 
    VV_CE = log(CC_CE) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = log(c_bar_CE) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
else
    VV_CE = CC_CE.^(1-sigma)./(1-sigma) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = c_bar_CE.^(1-sigma)./(1-sigma) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
end
WW_CE = (1-al)*YY_CE./LL_CE;
KK_CE = al*(eta-1)/eta*YY_CE./rho;
ZZ_CE = YY_CE./(KK_CE.^al.*LL_CE.^(1-al));
y_bar_CE = Theta*x_bar_CE.^ga.*l_bar_CE.^(1-ga);
w_bar_CE = (1-al)*y_bar_CE./l_bar_CE;
k_bar_CE = al*(eta-1)/eta*y_bar_CE./rho;
Z_bar_CE = y_bar_CE./(k_bar_CE.^al.*l_bar_CE.^(1-al));


%%%%%%%%%%%%%%%%%%%%%
% PLANNER'S PROBLEM %
%%%%%%%%%%%%%%%%%%%%%

mu_bar = mu_bar0;
mu_min = mu_min_0;
mu_max = mu_max_0;

maxit_tau = 40;
crit = 10^(-6);
% T1 = 120;
T1 = 70;
N1 = round(T1/dt);
% maxit = 1;
% mu_bar = .2;
for i=1:maxit

%SEPARABLE UTILITY
x_bar = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar/a0*(1-tau_bar)*(1-al)*Theta)^(1/phi);
l_bar = (mu_bar/a0*(1-tau_bar)*(1-al)*Theta*x_bar^ga).^(1/(phi+ga));
c_bar = mu_bar^(-1/sigma); %SEPARABLE

%x0 = .1*x_bar; %INITIAL WEALTH OF ENTREPRENEURS
%x0 = x_bar; %INITIAL WEALTH OF ENTREPRENEURS

tau0 = -0.17;
tau_max = tau_bar;
% tau_min = -0.5;
tau_min = -0.6;

%SHOOTING ALGORITHM FINDING tau0
for j=1:maxit_tau
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N-1
    l_t(n) = (mu_bar/a0.*(max(1-tau_t(n),0))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(max(1-tau_t(n),0))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
    x_t(n+1) = max(x_t(n+1),0.000001);
end


tau_LR(j) = tau_t(N1)-tau_t(N1-1);
tau_min_j(j) = tau_min;
tau_max_j(j) = tau_max;
tau0_j(j) = tau0;

tau_t_save(:,j) = tau_t;

%UPDATE tau0
if tau_LR(j)>crit
    %disp('tau_LR>0')
    tau_max = tau0;
    tau0 = 0.5*(tau0+tau_min);
elseif tau_LR(j)<-crit;
    %disp('tau_LR<0')
    tau_min = tau0;
    tau0 = 0.5*(tau0+tau_max);
elseif abs(tau_LR(j))<crit;
    display('Shooting algorithm converged, tau0 =')
    disp(tau0)
    break
end

if j==maxit_tau
    disp('Shooting algorithm did not converge fully, tau0 = ')
    disp(tau0)
    %If algorithm doesn't converge, take the tau0 that works the best out
    %of all tau0s
    [obj index] = min(abs(tau_LR(1:j)));
    tau0 = tau0_j(index);
end

end

%plot([tau_min_j',tau0_j',tau_max_j'])

%RUN SYSTEM ONE MORE TIME TO GET NICE TIME PATHS FOR tau and x.
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N
    l_t(n) = (mu_bar/a0.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*al/eta*(1-tau_t(n))*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    if n>N1
        tau_t(n+1) = tau_t(n);
    end
    dXX(n) = al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end
x_t(N+1)=[];
tau_t(N+1)=[];

YY = Theta*x_t.^ga.*l_t.^(1-ga);
yPDV = sum(dt*YY.*exp(-rho.*time)) + exp(-rho*T)*YY(N)/rho;

F(i) = b0 + (1-al)*yPDV - c_bar/rho;
mu_i(i)=mu_bar;

%UPDATE INTEREST RATE
if F(i)>crit
    disp('F>0')
    mu_max = mu_bar;
    mu_bar = 0.5*(mu_bar+mu_min);
elseif F(i)<-crit;
    disp('F<0')
    mu_min = mu_bar;
    mu_bar = 0.5*(mu_bar+mu_max);
elseif abs(F(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort index] = sort(mu_i);
F_sort = F(index);
plot(mu_sort,F_sort,mu_sort,zeros(1,length(F)),'--')

%COMPUTE AND PLOT SOME VARIABLES AND COMPARE TO COMPETITIVE EQUILIBRIUM
LL = l_t;
XX = x_t;
TAU = tau_t;
CC = c_bar*ones(1,N);
if sigma == 1 
    VV = log(CC) - a0*LL.^(1+phi)./(1+phi); %Welfare
else
    VV = CC.^(1-sigma)./(1-sigma) - a0*LL.^(1+phi)./(1+phi); %Welfare
end
WW = (1-al)*YY./LL;
KK = al*(eta-1)/eta*YY./rho;
ZZ = YY./(KK.^al.*LL.^(1-al));
y_bar = Theta*x_bar.^ga.*l_bar.^(1-ga);
w_bar = (1-al)*y_bar./l_bar;
k_bar = al*(eta-1)/eta*y_bar./rho;
Z_bar = y_bar./(k_bar.^al.*l_bar.^(1-al));

%TIME ZERO WELFARE, NO TRANSFERS
%WELF0 = V_bar/rho;
WELF0 = sum(dt*VV.*exp(-rho.*time)) + exp(-rho*T)*VV(N)/rho;
WELF0_CE = sum(dt*VV_CE.*exp(-rho.*time)) + exp(-rho*T)*VV_CE(N)/rho;
if sigma ~=1
CWELF0 = sum(dt*CC.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC(N).^(1-sigma)/(1-sigma)/rho;
CWELF0_CE = sum(dt*CC_CE.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC_CE(N).^(1-sigma)/(1-sigma)/rho;
LWELF0 = sum(dt*a0*LL.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL(N).^(1+phi)/(1+phi)/rho;
LWELF0_CE = sum(dt*a0*LL_CE.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL_CE(N).^(1+phi)/(1+phi)/rho;
end

%PERIOD BY PERIOD CONSUMPTION EQUIVALENT
VV_diff = VV - VV_CE;
CEQ_t = exp(VV_diff) -1;

%TIME ZERO CONSUMPTION EQUIVALENT
WELF0_diff = WELF0 - WELF0_CE;
CEQ0 = exp(rho*WELF0_diff) -1;

if sigma ~=1
   CEQ0 = ((CWELF0 - LWELF0 + LWELF0_CE)/CWELF0_CE).^(1/(1-sigma))-1;
end


%Make sure that CE/planner variables are not used elsewhere by deleting all
%unnecessary variables
clearvars -except x0 b0 x_bar_CE TAU XX XX_CE LL LL_CE ZZ ZZ_CE WW WW_CE VV VV_CE YY YY_CE KK KK_CE mu_bar0 mu_min_0 mu_max_0 T N dt time maxit maxit_tau Smax al eta ga Theta phi sigma d rho a0 CEQ0 CEQ_t WELF0_CE CWELF0_CE LWELF0_CE


%%%%%%%%%%%%%%%%%%%%%
% LIMITED TRANSFERS %
%%%%%%%%%%%%%%%%%%%%%

mu_bar_Tr = mu_bar0;
mu_min_Tr = mu_min_0;
mu_max_Tr = mu_max_0;

maxit_tau = 40;
crit = 10^(-6);
T1 = 50;
N1 = round(T1/dt);

% maxit = 1;
% mubar_Tr = 0.2;
for i=1:maxit
    
%STEADY STATE WITH TRANSFERS
x_bar_Tr = (al/eta*Theta/d)^((phi+ga)/(phi*(1-ga)))*(mu_bar_Tr*(1-al)*Theta)^(1/phi);
l_bar_Tr = (mu_bar_Tr*(1-al)*Theta*x_bar_Tr^ga).^(1/(phi+ga));
c_bar_Tr = mu_bar_Tr^(-1/sigma); %SEPARABLE
s_x_bar = -rho;
S_bar = s_x_bar*x_bar_Tr;

tau0 = -0.17;
tau_max = -0.1;
tau_min = -0.8;

%SHOOTING ALGORITHM FINDING tau0
for j=1:maxit_tau
x_t(1) = x0;
tau_t(1) = tau0;
% for n=1:N-1
%     l_t(n) = (mu_bar_Tr.*(max(1-tau_t(n),0))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
%     xxx = ga*(1-tau_t(n))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
%     tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
%     S_t(n) = (tau_t(n)<0)*Smax + (1-(tau_t(n)<0))*S_bar;
%     x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n) + S_t(n)) + x_t(n);
%     x_t(n+1) = max(x_t(n+1),.0001);
% end


for n=1:N-1
    l_t(n) = (mu_bar_Tr/a0.*(max(1-tau_t(n),0))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(max(1-tau_t(n),0))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    S_t(n) = (tau_t(n)<0)*Smax + (1-(tau_t(n)<0))*S_bar;
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n) + S_t(n)) + x_t(n);
    x_t(n+1) = max(x_t(n+1),0.000001);
end


tau_LR(j) = tau_t(N1)-tau_t(N1-1);
tau_min_j(j) = tau_min;
tau_max_j(j) = tau_max;
tau0_j(j) = tau0;

tau_t_save(:,j) = tau_t;

%UPDATE tau0
if tau_LR(j)>crit
    %disp('tau_LR>0')
    tau_max = tau0;
    tau0 = 0.5*(tau0+tau_min);
elseif tau_LR(j)<-crit;
    %disp('tau_LR<0')
    tau_min = tau0;
    tau0 = 0.5*(tau0+tau_max);
elseif abs(tau_LR(j))<crit;
    display('Shooting algorithm converged, tau0 =')
    disp(tau0)
    break
end

if j==maxit_tau
    disp('Shooting algorithm did not converge fully, tau0 =')
    disp(tau0)
    %If algorithm doesn't converge, take the tau0 that works the best out
    %of all tau0s
    [obj index] = min(abs(tau_LR(1:j)));
    tau0 = tau0_j(index);
end

end

%plot([tau_min_j',tau0_j',tau_max_j'])

%RUN SYSTEM ONE MORE TIME TO GET NICE TIME PATHS FOR tau and x.
x_t(1) = x0;
tau_t(1) = tau0;
% for n=1:N
%     l_t(n) = (mu_bar_Tr.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
%     xxx = ga*al/eta*(1-tau_t(n))*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
%     tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
%     if n>N1
%         tau_t(n+1) = tau_t(n);
%     end
%     S_t(n) = (tau_t(n)<0)*Smax + (1-(tau_t(n)<0))*S_bar;
%     x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n) + S_t(n)) + x_t(n);
% end
% x_t(N+1)=[];
% tau_t(N+1)=[];

for n=1:N
    l_t(n) = (mu_bar_Tr/a0.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*al/eta*(1-tau_t(n))*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    if n>N1
        tau_t(n+1) = tau_t(n);
    end
    S_t(n) = (tau_t(n)<0)*Smax + (1-(tau_t(n)<0))*S_bar;
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n) + S_t(n)) + x_t(n);
end
x_t(N+1)=[];
tau_t(N+1)=[];

%COMPUTE THE FUNCTION F
YY_Tr = Theta*x_t.^ga.*l_t.^(1-ga);
yPDV_Tr = sum(dt*YY_Tr.*exp(-rho.*time)) + exp(-rho*T)*YY_Tr(N)/rho;
F_Tr(i) = b0 + (1-al)*yPDV_Tr - c_bar_Tr/rho;
mu_i_Tr(i)=mu_bar_Tr;

%UPDATE INTEREST RATE
if F_Tr(i)>crit
    disp('F>0')
    mu_max_Tr = mu_bar_Tr;
    mu_bar_Tr = 0.5*(mu_bar_Tr+mu_min_Tr);
elseif F_Tr(i)<-crit;
    disp('F<0')
    mu_min_Tr = mu_bar_Tr;
    mu_bar_Tr = 0.5*(mu_bar_Tr+mu_max_Tr);
elseif abs(F_Tr(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_Tr)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort index] = sort(mu_i_Tr);
F_sort = F_Tr(index);
plot(mu_sort,F_sort,mu_sort,zeros(1,length(F_Tr)),'--')

LL_Tr = l_t;
XX_Tr = x_t;
TAU_Tr = tau_t;

CC_Tr = c_bar_Tr*ones(1,N);
if sigma == 1 
    VV_Tr = log(CC_Tr) - a0*LL_Tr.^(1+phi)./(1+phi); %Welfare
else
    VV_Tr = CC_Tr.^(1-sigma)./(1-sigma) - a0*LL_Tr.^(1+phi)./(1+phi); %Welfare
end
WW_Tr = (1-al)*YY_Tr./LL_Tr;
KK_Tr = al*(eta-1)/eta*YY_Tr./rho;
ZZ_Tr = YY_Tr./(KK_Tr.^al.*LL_Tr.^(1-al));

%PERIOD BY PERIOD CONSUMPTION EQUIVALENT
VV_diff_Tr = VV_Tr - VV_CE;
CEQ_Tr_t = exp(VV_diff_Tr) -1;

% plot(time,LL_CE,time,LL_Tr)
% plot(time,CEQ_Tr_t)
% break



%%%%%%%%%%%%%%%%%%%%
% FIGURE FOR PAPER %
%%%%%%%%%%%%%%%%%%%%
Tmax = 80;
figure(1)
set(gcf,'PaperPosition',[0 0 15 5])
subplot(1,2,1)
h1 = plot(time,zeros(1,N),time,TAU,'--',time,TAU_Tr,'r','LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, No Transf.','Planner, Lim. Transf.','Location','SouthEast')
xlim([0 Tmax])
title('(a) Labor Tax, $\tau_\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(1,2,2)
h1 = plot(time,XX_CE/x_bar_CE,time,XX/x_bar_CE,'--',time,XX_Tr/x_bar_CE,'r','LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, No Transf.','Planner, Lim. Transf.','Location','SouthEast')
xlim([0 Tmax])
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')
print -depsc graphs/appendix/fig5.eps

figure(2)
set(gcf,'PaperPosition',[0 0 15 18])
subplot(3,2,1)
h1 = plot(time,zeros(1,N),time,(LL-LL_CE)./LL_CE,'--',time,(LL_Tr-LL_CE)./LL_CE,'LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(a) Labor Supply, $\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,2)
h1 = plot(time,zeros(1,N),time,(XX-XX_CE)./XX_CE,'--',time,(XX_Tr-XX_CE)./XX_CE,'LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,3)
h1 = plot(time,zeros(1,N),time,(WW-WW_CE)./WW_CE,'--',time,(WW_Tr-WW_CE)./WW_CE,'LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(c) Wage, $w$, and Labor Productivity, $y/\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,4)
h1 = plot(time,zeros(1,N),time,(ZZ-ZZ_CE)./ZZ_CE,'--',time,(ZZ_Tr-ZZ_CE)./ZZ_CE,'LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(d) Total Factor Productivity, $Z$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,5)
h1 = plot(time,zeros(1,N),time,(YY-YY_CE)./YY_CE,'--',time,(YY_Tr-YY_CE)./YY_CE,'LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(e) Income, $y$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,6)
h1 = plot(time,zeros(1,N),time,CEQ_t,'--',time,CEQ_Tr_t,'LineWidth',2);
set(h1(1),'linewidth',.5);
set(gca,'FontSize',16)
xlim([0 Tmax])
title('(f) Worker Period Utility, $u(c,\ell)$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

print -depsc graphs/extras/fig6.eps

clearvars -except VV_Tr CC_Tr LL_Tr VV_CE WELF0_CE CWELF0_CE LWELF0_CE time dt sigma phi rho a0 T N CEQ0

%TIME ZERO WELFARE, LIMITED TRANSFERS
WELF0_Tr = sum(dt*VV_Tr.*exp(-rho.*time)) + exp(-rho*T)*VV_Tr(N)/rho;
if sigma ~=1
CWELF0_Tr = sum(dt*CC_Tr.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC_Tr(N).^(1-sigma)/(1-sigma)/rho;
LWELF0_Tr = sum(dt*a0*LL_Tr.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL_Tr(N).^(1+phi)/(1+phi)/rho;
end

%TIME ZERO CONSUMPTION EQUIVALENT
WELF0_diff_Tr = WELF0_Tr - WELF0_CE;
CEQ0_Tr = exp(rho*WELF0_diff_Tr) -1;

if sigma ~=1
   CEQ0_Tr = ((CWELF0_Tr - LWELF0_Tr + LWELF0_CE)/CWELF0_CE).^(1/(1-sigma))-1;
end

disp('Welfare Gain, No Transfers (Consumption Equivalents) = ')
disp(CEQ0)

disp('Welfare Gain, Limited Transfers (Consumption Equivalents) = ')
disp(CEQ0_Tr)