clear all; clc;
close all;

% Parameters
eta = 1.06;                % Pareto shape parameter of productivity distribution
al = 1/3;               % capital share in production
rho = 0.03;               % world interest rate
d = 0.1;                % impatience of entrepreneurs
la = 2;                % financial friction
A = 1;                  %productivity
%PREFERENCE PARAMETERS
sigma = 1; Frisch = 0.5; phi = 1/Frisch; a0=1;

theta = 1; %PARETO WEIGHT

ga = (al/eta)/(al/eta +(1-al));
Theta = (rho/al)*(eta*la/(eta-1)*(al*A/rho)^(eta/al))^ga;

tau_bar = ga/(ga + (1-ga)*(d/rho));

b0 = 1; %INITIAL BOND HOLDINGS OF WORKERS

%FIND mu_bar that solves system
mu_min_0 = .0001;
mu_max_0 = 1;
mu_bar0 = .6;

maxit = 30;
crit = 10^(-5);

T = 150;
N = 10000;
dt = T/N;
time = dt*(1:N);


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPETITIVE EQUILIBRIUM %
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIND mu_bar that solves system
mu_bar_CE = mu_bar0;
mu_min_CE = mu_min_0;
mu_max_CE = mu_max_0;

for i=1:maxit

%SEPARABLE UTILITY
x_bar_CE = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar_CE*(1-al)*Theta)^(1/phi);
l_bar_CE = (mu_bar_CE*(1-al)*Theta*x_bar_CE^ga).^(1/(phi+ga));
c_bar_CE = mu_bar_CE^(-1/sigma); %SEPARABLE

x0 = .2*x_bar_CE; %INITIAL WEALTH OF ENTREPRENEURS
%x0 = .1*x_bar_CE; %INITIAL WEALTH OF ENTREPRENEURS

XX_CE(1) = x0;
for n=1:N
    LL_CE(n) = (mu_bar_CE.*(1-al)*Theta*XX_CE(n)^ga).^(1/(phi+ga));
    dXX_CE(n) = al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n);
    XX_CE(n+1)  = dt*(al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n)) + XX_CE(n);
end
XX_CE(N+1)=[];

YY_CE = Theta*XX_CE.^ga.*LL_CE.^(1-ga);
yPDV_CE = sum(dt*YY_CE.*exp(-rho.*time)) + exp(-rho*T)*YY_CE(N)/rho;

F_CE(i) = b0 + (1-al)*yPDV_CE - c_bar_CE/rho;
mu_i_CE(i)=mu_bar_CE;

%UPDATE INTEREST RATE
if F_CE(i)>crit
    disp('F>0')
    mu_max_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_min_CE);
elseif F_CE(i)<-crit;
    disp('F<0')
    mu_min_CE = mu_bar_CE;
    mu_bar_CE = 0.5*(mu_bar_CE+mu_max_CE);
elseif abs(F_CE(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_CE)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort_CE index_CE] = sort(mu_i_CE);
F_sort_CE = F_CE(index_CE);
close all
plot(mu_sort_CE,F_sort_CE,mu_sort_CE,zeros(1,length(F_CE)),'--')

CC_CE = c_bar_CE*ones(1,N);
if sigma == 1 
    VV_CE = log(CC_CE) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = log(c_bar_CE) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
else
    VV_CE = CC_CE.^(1-sigma)./(1-sigma) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = c_bar_CE.^(1-sigma)./(1-sigma) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
end
WW_CE = (1-al)*YY_CE./LL_CE;
KK_CE = al*(eta-1)/eta*YY_CE./rho;
ZZ_CE = YY_CE./(KK_CE.^al.*LL_CE.^(1-al));
y_bar_CE = Theta*x_bar_CE.^ga.*l_bar_CE.^(1-ga);
w_bar_CE = (1-al)*y_bar_CE./l_bar_CE;
k_bar_CE = al*(eta-1)/eta*y_bar_CE./rho;
Z_bar_CE = y_bar_CE./(k_bar_CE.^al.*l_bar_CE.^(1-al));


%%%%%%%%%%%%%%%%%%%%%
% PLANNER'S PROBLEM %
%%%%%%%%%%%%%%%%%%%%%

mu_bar = mu_bar0;
mu_min = mu_min_0;
mu_max = mu_max_0;

maxit_tau = 40;
crit = 10^(-6);
T1 = 80;
N1 = round(T1/dt);

for i=1:maxit

%SEPARABLE UTILITY
x_bar = (al/eta*Theta/(d-rho))^((phi+ga)/(phi*(1-ga)))*(mu_bar*(1-tau_bar)*(1-al)*Theta)^(1/phi);
l_bar = (mu_bar*(1-tau_bar)*(1-al)*Theta*x_bar^ga).^(1/(phi+ga));
c_bar = mu_bar^(-1/sigma); %SEPARABLE

tau0 = -0.17;
tau_max = -0.05;
% tau_min = -0.5;
tau_min = -0.6;

%SHOOTING ALGORITHM FINDING tau0
for j=1:maxit_tau
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N-1
    l_t(n) = (mu_bar_CE.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(1-tau_t(n))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end


tau_LR(j) = tau_t(N1)-tau_t(N1-1);
tau_min_j(j) = tau_min;
tau_max_j(j) = tau_max;
tau0_j(j) = tau0;

tau_t_save(:,j) = tau_t;

%UPDATE tau0
if tau_LR(j)>crit
    %disp('tau_LR>0')
    tau_max = tau0;
    tau0 = 0.5*(tau0+tau_min);
elseif tau_LR(j)<-crit;
    %disp('tau_LR<0')
    tau_min = tau0;
    tau0 = 0.5*(tau0+tau_max);
elseif abs(tau_LR(j))<crit;
    display('Shooting algorithm converged, tau0 =')
    disp(tau0)
    break
end

if j==maxit_tau
    disp('Shooting algorithm did not converge fully, tau0 = ')
    disp(tau0)
    %If algorithm doesn't converge, take the tau0 that works the best out
    %of all tau0s
    [obj index] = min(abs(tau_LR(1:j)));
    tau0 = tau0_j(index);
end

end

%plot([tau_min_j',tau0_j',tau_max_j'])

%RUN SYSTEM ONE MORE TIME TO GET NICE TIME PATHS FOR tau and x.
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N
    l_t(n) = (mu_bar_CE.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*al/eta*(1-tau_t(n))*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    if n>N1
        tau_t(n+1) = tau_t(n);
    end
    dXX(n) = al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end
x_t(N+1)=[];
tau_t(N+1)=[];

YY = Theta*x_t.^ga.*l_t.^(1-ga);
yPDV = sum(dt*YY.*exp(-rho.*time)) + exp(-rho*T)*YY(N)/rho;

F(i) = b0 + (1-al)*yPDV - c_bar/rho;
mu_i(i)=mu_bar;

%UPDATE INTEREST RATE
if F(i)>crit
    disp('F>0')
    mu_max = mu_bar;
    mu_bar = 0.5*(mu_bar+mu_min);
elseif F(i)<-crit;
    disp('F<0')
    mu_min = mu_bar;
    mu_bar = 0.5*(mu_bar+mu_max);
elseif abs(F(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort index] = sort(mu_i);
F_sort = F(index);
plot(mu_sort,F_sort,mu_sort,zeros(1,length(F)),'--')

%COMPUTE AND PLOT SOME VARIABLES AND COMPARE TO COMPETITIVE EQUILIBRIUM
LL = l_t;
XX = x_t;
TAU = tau_t;
CC = c_bar*ones(1,N);
if sigma == 1 
    VV = log(CC) - a0*LL.^(1+phi)./(1+phi); %Welfare
else
    VV = CC.^(1-sigma)./(1-sigma) - a0*LL.^(1+phi)./(1+phi); %Welfare
end
WW = (1-al)*YY./LL;
KK = al*(eta-1)/eta*YY./rho;
ZZ = YY./(KK.^al.*LL.^(1-al));
y_bar = Theta*x_bar.^ga.*l_bar.^(1-ga);
w_bar = (1-al)*y_bar./l_bar;
k_bar = al*(eta-1)/eta*y_bar./rho;
Z_bar = y_bar./(k_bar.^al.*l_bar.^(1-al));




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% POSITIVE PARETO WEIGHT ON ENTREPRENEURS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu_bar_WE = mu_bar0;
mu_min_WE = mu_min_0;
mu_max_WE = mu_max_0;

maxit_tau = 40;
crit = 10^(-6);
T1 = 50;
N1 = round(T1/dt);

for i=1:maxit
    
%STEADY STATE WITH TRANSFERS
x_bar_WE = (al/eta*Theta/d)^((phi+ga)/(phi*(1-ga)))*(mu_bar_WE*(1-al)*Theta)^(1/phi);
l_bar_WE = (mu_bar_WE*(1-al)*Theta*x_bar_WE^ga).^(1/(phi+ga));
c_bar_WE = mu_bar_WE^(-1/sigma); %SEPARABLE
s_x_bar = -rho;
S_bar = s_x_bar*x_bar_WE;

tau0 = -0.17;
tau_max = -0.05;
tau_min = -0.8;

%SHOOTING ALGORITHM FINDING tau0
for j=1:maxit_tau
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N-1
    l_t(n) = (mu_bar_WE.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(1-tau_t(n))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1) + theta/mu_bar_WE*ga/x_t(n)*(al/eta*Theta*x_t(n)^(ga-1)*l_t(n)^(1-ga) + rho);
    tau_t(n+1) = dt*(d*(tau_t(n)-ga) + xxx) + tau_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end


tau_LR(j) = tau_t(N1)-tau_t(N1-1);
tau_min_j(j) = tau_min;
tau_max_j(j) = tau_max;
tau0_j(j) = tau0;

tau_t_save(:,j) = tau_t;

%UPDATE tau0
if tau_LR(j)>crit
    %disp('tau_LR>0')
    tau_max = tau0;
    tau0 = 0.5*(tau0+tau_min);
elseif tau_LR(j)<-crit;
    %disp('tau_LR<0')
    tau_min = tau0;
    tau0 = 0.5*(tau0+tau_max);
elseif abs(tau_LR(j))<crit;
    display('Shooting algorithm converged, tau0 =')
    disp(tau0)
    break
end

if j==maxit_tau
    disp('Shooting algorithm did not converge fully, tau0 =')
    disp(tau0)
    %If algorithm doesn't converge, take the tau0 that works the best out
    %of all tau0s
    [obj index] = min(abs(tau_LR(1:j)));
    tau0 = tau0_j(index);
end

end

%plot([tau_min_j',tau0_j',tau_max_j'])

%RUN SYSTEM ONE MORE TIME TO GET NICE TIME PATHS FOR tau and x.
x_t(1) = x0;
tau_t(1) = tau0;
for n=1:N
    l_t(n) = (mu_bar_WE.*(1-tau_t(n))*(1-al)*Theta*x_t(n)^ga).^(1/(phi+ga));
    xxx = ga*(1-tau_t(n))*al/eta*Theta*l_t(n)^(1-ga)*x_t(n)^(ga-1) + theta/mu_bar_WE*ga/x_t(n)*(al/eta*Theta*x_t(n)^(ga-1)*l_t(n)^(1-ga) + rho);
    if n>N1
        tau_t(n+1) = tau_t(n);
    end
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end
x_t(N+1)=[];
tau_t(N+1)=[];

%COMPUTE THE FUNCTION F
YY_WE = Theta*x_t.^ga.*l_t.^(1-ga);
yPDV_WE = sum(dt*YY_WE.*exp(-rho.*time)) + exp(-rho*T)*YY_WE(N)/rho;
F_WE(i) = b0 + (1-al)*yPDV_WE - c_bar_WE/rho;
mu_i_WE(i)=mu_bar_WE;

%UPDATE INTEREST RATE
if F_WE(i)>crit
    disp('F>0')
    mu_max_WE = mu_bar_WE;
    mu_bar_WE = 0.5*(mu_bar_WE+mu_min_WE);
elseif F_WE(i)<-crit;
    disp('F<0')
    mu_min_WE = mu_bar_WE;
    mu_bar_WE = 0.5*(mu_bar_WE+mu_max_WE);
elseif abs(F_WE(i))<crit;
    display('Equilibrium Found, mu_bar =')
    disp(mu_bar_WE)
    break
end

end

%CHECK FOR MONOTONICITY OF F
[mu_sort index] = sort(mu_i_WE);
F_sort = F_WE(index);
plot(mu_sort,F_sort,mu_sort,zeros(1,length(F_WE)),'--')

LL_WE = l_t;
XX_WE = x_t;
TAU_WE = tau_t;

CC_WE = c_bar_WE*ones(1,N);
if sigma == 1 
    VV_WE = log(CC_WE) - a0*LL_WE.^(1+phi)./(1+phi); %Welfare
else
    VV_WE = CC_WE.^(1-sigma)./(1-sigma) - a0*LL_WE.^(1+phi)./(1+phi); %Welfare
end
WW_WE = (1-al)*YY_WE./LL_WE;
KK_WE = al*(eta-1)/eta*YY_WE./rho;
ZZ_WE = YY_WE./(KK_WE.^al.*LL_WE.^(1-al));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NORMALIZE VARIABLES BY CE STEADY STATE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LL = LL./l_bar_CE; LL_CE = LL_CE./l_bar_CE; LL_WE = LL_WE./l_bar_CE;
WW = WW./w_bar_CE; WW_CE = WW_CE./w_bar_CE; WW_WE = WW_WE./w_bar_CE;
XX = XX./x_bar_CE; XX_CE = XX_CE./x_bar_CE; XX_WE = XX_WE./x_bar_CE;
KK = KK./k_bar_CE; KK_CE = KK_CE./k_bar_CE; KK_WE = KK_WE./k_bar_CE;
ZZ = ZZ./Z_bar_CE; ZZ_CE = ZZ_CE./Z_bar_CE; ZZ_WE = ZZ_WE./Z_bar_CE;
YY = YY./y_bar_CE; YY_CE = YY_CE./y_bar_CE; YY_WE = YY_WE./y_bar_CE;
VV = VV./V_bar_CE; VV_CE = VV_CE./V_bar_CE; VV_WE = VV_WE./V_bar_CE;

% %%%%%%%%%%%%%%%%%%%%%
% % PLOTTING FIGURE 4 %
% %%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%
% FIGURE FOR PAPER %
%%%%%%%%%%%%%%%%%%%%
figure(2)
set(gcf,'PaperPosition',[0 0 15 20])
subplot(4,2,1)
h1 = plot(time,zeros(1,N),'--',time,TAU,'-.',time,TAU_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
% ylim([-0.05 0.15])
title('(a) Labor Tax','interpreter','latex')
ylabel('Labor Tax, $\tau_\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,2)
h1 = plot(time,LL_CE,'--',time,LL,'-.',time,LL_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
title('(b) Labor Supply','interpreter','latex')
ylabel('Labor Supply, $\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,3)
h1 = plot(time,WW_CE,'--',time,WW,'-.',time,WW_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
title('(c) Wage/Labor Productivity','interpreter','latex')
ylabel('Wage $w$/Labor Productivity $y/\ell$','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,4)
h1 = plot(time,XX_CE,'--',time,XX,'-.',time,XX_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
title('(d) Entrepreneurial Wealth','interpreter','latex')
ylabel('Entrepreneurial Wealth','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,5)
h1 = plot(time,KK_CE,'--',time,KK,'-.',time,KK_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
title('(e) Capital','interpreter','latex')
ylabel('Capital','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,6)
h1 = plot(time,ZZ_CE,'--',time,ZZ,'-.',time,ZZ_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
title('(f) Total Factor Productivity','interpreter','latex')
ylabel('TFP, $Z$','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,7)
h1 = plot(time,YY_CE,'--',time,YY,'-.',time,YY_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','SouthEast')
% xlim([-0.5 30])
title('(g) GDP','interpreter','latex')
ylabel('GDP, $y$','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(4,2,8)
h1 = plot(time,VV_CE,'--',time,VV,'-.',time,VV_WE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner, \theta=0','Planner, \theta>0','Location','NorthEast')
% xlim([-0.5 30])
title('(h) Worker Period Utility','interpreter','latex')
ylabel('Worker Period Utility','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

print -depsc graphs/extras/figA1.eps

