clear all; clc; close all;

global al Theta sigma ga phi a0 x_current nu_current

% Parameters
eta = 1.06;                % Pareto shape parameter of productivity distribution
al = 1/3;               % capital share in production
rho = 0.03;               % world interest rate
d = 0.1;                % impatience of entrepreneurs
la = 2;                % financial friction
A = 1;                  %productivity
%PREFERENCE PARAMETERS
sigma = 1;
Frisch = 1;
phi = 1/Frisch; a0=1;

%COMMENT: If we can somehow defend d=0.13 and Frisch=0.3, the figures look
%even nicer

%AUXILIARY PARAMETERS
ga = (al/eta)/(al/eta +(1-al));
Theta = (rho/al)*(eta*la/(eta-1)*(al*A/rho)^(eta/al))^ga;

tau_bar = ga/(ga + (1-ga)*(d/rho));

b0 = 0; %INITIAL BOND HOLDINGS OF WORKERS

%FIND mu_bar that solves system
mu_min_0 = .0001;
mu_max_0 = 1;
mu_bar0 = .6;

maxit = 30;
crit = 10^(-5);

T = 150;
N = 3000;
% T = 200;
% N = 15000;
dt = T/N;
time = dt*(1:N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPETITIVE EQUILIBRIUM %
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIND mu_bar that solves system
mu_bar_CE = mu_bar0;
mu_min_CE = mu_min_0;
mu_max_CE = mu_max_0;


%SEPARABLE UTILITY

term1 = ((al/eta)*(Theta/(d-rho)))^((phi+ga + sigma*(1-ga))/((1-ga)*(phi + sigma)));
term2 = ((1/a0)*((1-al)*Theta)^(1-sigma))^(1/(phi+sigma));
x_bar_CE = term1*term2;
l_bar_CE = ((1/a0)*((1-al)*Theta)^(1-sigma)*x_bar_CE^(ga*(sigma-1)))^(1/(phi+ga+sigma*(1-ga)));
%check = (al/eta)*Theta*x_bar_CE^(ga-1)*l_bar_CE^(1-ga) + rho - d
y_bar_CE = Theta*x_bar_CE^ga*l_bar_CE^(1-ga);
c_bar_CE =(1-al)*y_bar_CE; %SEPARABLE

x0 = .1*x_bar_CE; %INITIAL WEALTH OF ENTREPRENEURS

XX_CE(1) = x0;
for n=1:N
    LL_CE(n) = ((1/a0)*((1-al)*Theta)^(1-sigma)*XX_CE(n)^(ga*(sigma-1)))^(1/(phi+ga+sigma*(1-ga)));
    XX_CE(n+1)  = dt*(al/eta*Theta*XX_CE(n)^ga.*LL_CE(n)^(1-ga) + (rho-d)*XX_CE(n)) + XX_CE(n);
end
XX_CE(N+1)=[];

YY_CE = Theta*XX_CE.^ga.*LL_CE.^(1-ga);
CC_CE = (1-al)*YY_CE;

if sigma == 1 
    VV_CE = log(CC_CE) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = log(c_bar_CE) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
else
    VV_CE = CC_CE.^(1-sigma)./(1-sigma) - a0*LL_CE.^(1+phi)./(1+phi); %Welfare
    V_bar_CE = c_bar_CE.^(1-sigma)./(1-sigma) - a0*l_bar_CE.^(1+phi)./(1+phi); %Welfare
end
WW_CE = (1-al)*YY_CE./LL_CE;
KK_CE = al*(eta-1)/eta*YY_CE./rho;
ZZ_CE = YY_CE./(KK_CE.^al.*LL_CE.^(1-al));
y_bar_CE = Theta*x_bar_CE.^ga.*l_bar_CE.^(1-ga);
w_bar_CE = (1-al)*y_bar_CE./l_bar_CE;
k_bar_CE = al*(eta-1)/eta*y_bar_CE./rho;
Z_bar_CE = y_bar_CE./(k_bar_CE.^al.*l_bar_CE.^(1-al));

%plot(time,XX_CE,time,x_bar_CE*ones(1,N),'--')

%%%%%%%%%%%%%%%%%%%%%
% PLANNER'S PROBLEM %
%%%%%%%%%%%%%%%%%%%%%

maxit_nu = 30;
crit = 10^(-6);
% T1 = 120;
T1 = 70;
N1 = round(T1/dt);

% tau0 = -0.17;
% tau_max = 0.2;
% tau_min = -0.6;
% nu0 = (1 - tau0/ga)*c_bar_CE^(-sigma);
% nu_min = (1 - tau_max/ga)*c_bar_CE^(-sigma);
% nu_max = (1 - tau_min/ga)*c_bar_CE^(-sigma);

nu0 = 0.35;
nu_min = 0.3;
nu_max = 0.45;

%SHOOTING ALGORITHM FINDING tau0
for j=1:maxit_nu
x_t(1) = x0;
nu_t(1) = nu0;

disp('Iteration on nu0 = ')
disp(j)
disp('nu0 = ')
disp(nu0)

for n=1:N-1
    x_current = x_t(n);
    nu_current = nu_t(n);
    l0 = ((1/a0)*((1-al)*Theta)^(1-sigma)*XX_CE(n)^(ga*(sigma-1)))^(1/(phi+ga+sigma*(1-ga)));
    options=optimset('Display','off');   % Option to display output
    [l_t(n),fval,exitflag] = fsolve('labor_supply',l0,options);
    %[l_t(n),fval,exitflag] = fsolve('labor_supply',l0);
    y_t(n) = Theta*x_t(n)^ga*l_t(n)^(1-ga);
    c_t(n) = (1-al)*y_t(n);
    tau_t(n) = ga - ga*nu_t(n)*c_t(n)^sigma; 
    xxx = ((1-ga)*c_t(n)^(-sigma) + ga*nu_t(n))*(al/eta)*Theta*x_t(n)^(ga-1)*l_t(n)^(1-ga);
    nu_t(n+1) = dt*(d*nu_t(n) - xxx) + nu_t(n);
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
    x_t(n+1) = max(x_t(n+1),0.000001);
end


nu_LR(j) = nu_t(N1)-nu_t(N1-1);
nu_min_j(j) = nu_min;
nu_max_j(j) = nu_max;
nu0_j(j) = nu0;

nu_t_save(:,j) = nu_t;

%UPDATE tau0
if nu_LR(j)>crit
    %disp('nu_LR>0')
    nu_max = nu0;
    nu0 = 0.5*(nu0+nu_min);
elseif nu_LR(j)<-crit;
    %disp('nu_LR<0')
    nu_min = nu0;
    nu0 = 0.5*(nu0+nu_max);
elseif abs(nu_LR(j))<crit;
    display('Shooting algorithm converged, nu0 =')
    disp(nu0)
    break
end

if j==maxit_nu
    disp('Shooting algorithm did not converge fully, nu0 = ')
    disp(nu0)
    %If algorithm doesn't converge, take the tau0 that works the best out
    %of all tau0s
    [obj index] = min(abs(nu_LR(1:j)));
    nu0 = nu0_j(index);
end

end

%plot([tau_min_j',tau0_j',tau_max_j'])

%RUN SYSTEM ONE MORE TIME TO GET NICE TIME PATHS FOR tau and x.
x_t(1) = x0;
nu_t(1) = nu0;
for n=1:N
    x_current = x_t(n);
    nu_current = nu_t(n);
    l0 = ((1/a0)*((1-al)*Theta)^(1-sigma)*XX_CE(n)^(ga*(sigma-1)))^(1/(phi+ga+sigma*(1-ga)));
    options=optimset('Display','off');   % Option to display output
    [l_t(n),fval,exitflag] = fsolve('labor_supply',l0,options);
    %[l_t(n),fval,exitflag] = fsolve('labor_supply',l0);
    y_t(n) = Theta*x_t(n)^ga*l_t(n)^(1-ga);
    c_t(n) = (1-al)*y_t(n);
    tau_t(n) = ga - ga*nu_t(n)*c_t(n)^sigma; 
    xxx = ((1-ga)*c_t(n)^(-sigma) + ga*nu_t(n))*(al/eta)*Theta*x_t(n)^(ga-1)*l_t(n)^(1-ga);
    nu_t(n+1) = dt*(d*nu_t(n) - xxx) + nu_t(n);
    if n>N1
        nu_t(n+1) = nu_t(n);
    end
    x_t(n+1)  = dt*(al/eta*Theta*x_t(n)^ga.*l_t(n)^(1-ga) + (rho-d)*x_t(n)) + x_t(n);
end
x_t(N+1)=[];
nu_t(N+1)=[];

%COMPUTE AND PLOT SOME VARIABLES AND COMPARE TO COMPETITIVE EQUILIBRIUM
LL = l_t;
XX = x_t;
TAU = tau_t;
YY = y_t;
CC = (1-al)*YY;
if sigma == 1 
    VV = log(CC) - a0*LL.^(1+phi)./(1+phi); %Welfare
else
    VV = CC.^(1-sigma)./(1-sigma) - a0*LL.^(1+phi)./(1+phi); %Welfare
end
WW = (1-al)*YY./LL;
KK = al*(eta-1)/eta*YY./rho;
ZZ = YY./(KK.^al.*LL.^(1-al));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTING SOME NUMBERS OF INTEREST %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Largest difference in entrepreneurial wealth = ')
disp(max((XX - XX_CE)./XX_CE))
disp('Largest difference in Labor Supply = ')
disp(max((LL - LL_CE)./LL_CE))
disp('Largest difference in Wage = ')
disp(min((WW - WW_CE)./WW_CE))
disp('Largest difference in GDP = ')
disp(max((YY - YY_CE)./YY_CE))

%TIME ZERO WELFARE
%WELF0 = V_bar/rho;
WELF0 = sum(dt*VV.*exp(-rho.*time)) + exp(-rho*T)*VV(N)/rho;
WELF0_CE = sum(dt*VV_CE.*exp(-rho.*time)) + exp(-rho*T)*VV_CE(N)/rho;
if sigma ~=1
CWELF0 = sum(dt*CC.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC(N).^(1-sigma)/(1-sigma)/rho;
CWELF0_CE = sum(dt*CC_CE.^(1-sigma)/(1-sigma).*exp(-rho.*time)) + exp(-rho*T)*CC_CE(N).^(1-sigma)/(1-sigma)/rho;
LWELF0 = sum(dt*a0*LL.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL(N).^(1+phi)/(1+phi)/rho;
LWELF0_CE = sum(dt*a0*LL_CE.^(1+phi)/(1+phi).*exp(-rho.*time)) + exp(-rho*T)*a0*LL_CE(N).^(1+phi)/(1+phi)/rho;
end

%PERIOD BY PERIOD CONSUMPTION EQUIVALENT
VV_diff = VV - VV_CE;
CEQ_t = exp(VV_diff) -1;

%TIME ZERO CONSUMPTION EQUIVALENT
WELF0_diff = WELF0 - WELF0_CE;
CEQ0 = exp(rho*WELF0_diff) -1;

if sigma ~=1
   CEQ0 = ((CWELF0 - LWELF0 + LWELF0_CE)/CWELF0_CE).^(1/(1-sigma))-1;
end

disp('Welfare Gain (Consumption Equivalents) = ')
disp(CEQ0)


load data/fig2_3_save.mat

%%%%%%%%%%%%%%%%%%%%
% FIGURE FOR PAPER %
%%%%%%%%%%%%%%%%%%%%
Tmax = 80;
figure(1)
set(gcf,'PaperPosition',[0 0 15 10])
subplot(2,2,1)
h1 = plot(time,zeros(1,N),'--',time,TAU,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
% ylim([-0.05 0.15])
title('(a) Labor Tax','interpreter','latex')
ylabel('Labor Tax, $\tau_\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(2,2,2)
h1 = plot(time,XX_CE/x_bar_CE,'--',time,XX/x_bar_CE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(b) Entrepreneurial Wealth','interpreter','latex')
ylabel('Entrepreneurial Wealth','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')


subplot(2,2,3)
h1 = plot(time,CC_CE/c_bar_CE,'--',time,CC/x_bar_CE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','East')
xlim([0 Tmax])
title('(c) Consumption','interpreter','latex')
ylabel('Consumption','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')


subplot(2,2,4)
h1 = plot(time,LL_CE/x_bar_CE,'--',time,LL/x_bar_CE,'LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner','Location','NorthEast')
xlim([0 Tmax])
title('(d) Labor Supply','interpreter','latex')
ylabel('Labor Supply','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

print -depsc graphs/extras/handtomouth_levels.eps

figure(2)
set(gcf,'PaperPosition',[0 0 15 18])
subplot(3,2,1)
h1 = plot(time,zeros(1,N),'--',time,(XX-XX_CE)./XX_CE,'LineWidth',2);
set(gca,'FontSize',16)
%legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(a) Entrepreneurial Wealth Deviation','interpreter','latex')
ylabel('Entrepreneurial Wealth','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,2)
h1 = plot(time,zeros(1,N),'--',time,(LL-LL_CE)./LL_CE,'LineWidth',2);
set(gca,'FontSize',16)
%legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(b) Labor Supply','interpreter','latex')
ylabel('Labor Supply, $\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,3)
h1 = plot(time,zeros(1,N),'--',time,(WW-WW_CE)./WW_CE,'LineWidth',2);
set(gca,'FontSize',16)
%legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(c) Wage/Labor Productivity','interpreter','latex')
ylabel('Wage $w$/Labor Productivity $y/\ell$','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,4)
h1 = plot(time,zeros(1,N),'--',time,(KK-KK_CE)./KK_CE,'LineWidth',2);
set(gca,'FontSize',16)
%legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(d) Capital','interpreter','latex')
ylabel('Capital','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,5)
h1 = plot(time,zeros(1,N),'--',time,(YY-YY_CE)./YY_CE,'LineWidth',2);
set(gca,'FontSize',16)
%legend(h1,'Equilibrium','Planner','Location','SouthEast')
xlim([0 Tmax])
title('(e) GDP','interpreter','latex')
ylabel('GDP, $y$','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(3,2,6)
% h1 = plot(time,zeros(1,N),'--',time,(VV-VV_CE)./VV_CE,'LineWidth',2);
h1 = plot(time,zeros(1,N),'--',time,CEQ_t,'LineWidth',2);
set(gca,'FontSize',16)
%legend(h1,'Equilibrium','Planner','Location','NorthEast')
xlim([0 Tmax])
title('(f) Worker Period Utility','interpreter','latex')
ylabel('Worker Period Utility','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

print -depsc graphs/extras/handtomouth_deviations.eps

close all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIGURES WITH BOTH HtM AND BORROWING SAVING %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Tmax = 80;
figure(3)
set(gcf,'PaperPosition',[0 0 15 5])
subplot(1,2,1)
h1 = plot(time,zeros(1,N),'b--',time_bench,TAU_bench,'g',time,TAU,'r','LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium','Planner Benchmark','Planner HtM','Location','SouthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(a) Labor Tax, $\tau_\ell$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(1,2,2)
h1 = plot(time_bench,XX_CE_bench/x_bar_CE_bench,'b--',time_bench,XX_bench/x_bar_CE_bench,'g',time,XX_CE/x_bar_CE,'k--',time,XX/x_bar_CE,'r','LineWidth',2);
set(gca,'FontSize',16)
legend(h1,'Equilibrium Benchmark','Planner Benchmark','Equilibrium HtM','Planner HtM','Location','SouthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')
print -depsc graphs/extras/fig2_levels_comparison_htm.eps


figure(4)
set(gcf,'PaperPosition',[0 0 15 18])
subplot(3,2,1)
h1 = plot(time_bench,(LL_bench-LL_CE_bench)./LL_CE_bench,'g',time,(LL-LL_CE)./LL_CE,'r',time,zeros(1,N),'b--','LineWidth',2);
set(gca,'FontSize',14)
legend(h1,'Benchmark','Hand-to-Mouth','Location','NorthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(a) Labor Supply, $\ell$','interpreter','latex')
%ylabel('Labor Supply, $\ell$','interpreter','latex')
xlabel('Years','FontSize',14,'interpreter','latex')

subplot(3,2,2)
h1 = plot(time_bench,(XX_bench-XX_CE_bench)./XX_CE_bench,'g',time,(XX-XX_CE)./XX_CE,'r',time,zeros(1,N),'b--','LineWidth',2);
set(gca,'FontSize',14)
legend(h1,'Benchmark','Hand-to-Mouth','Location','NorthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(b) Entrepreneurial Wealth, $x$','interpreter','latex')
%ylabel('Entrepreneurial Wealth','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',14,'interpreter','latex')

subplot(3,2,3)
h1 = plot(time_bench,(WW_bench-WW_CE_bench)./WW_CE_bench,'g',time,(WW-WW_CE)./WW_CE,'r',time,zeros(1,N),'b--','LineWidth',2);
set(gca,'FontSize',14)
legend(h1,'Benchmark','Hand-to-Mouth','Location','SouthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(c) Wage, $w$, and Labor Productivity, $y/\ell$','interpreter','latex')
%ylabel('Wage $w$/Labor Productivity $y/\ell$','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',14,'interpreter','latex')

subplot(3,2,4)
h1 = plot(time_bench,(ZZ_bench-ZZ_CE_bench)./ZZ_CE_bench,'g',time,(ZZ-ZZ_CE)./ZZ_CE,'r',time,zeros(1,N),'b--','LineWidth',2);
set(gca,'FontSize',14)
legend(h1,'Benchmark','Hand-to-Mouth','Location','SouthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(d) Total Factor Productivity, $Z$','interpreter','latex')
xlabel('Years','FontSize',14,'interpreter','latex')

subplot(3,2,5)
h1 = plot(time_bench,(YY_bench-YY_CE_bench)./YY_CE_bench,'g',time,(YY-YY_CE)./YY_CE,'r',time,zeros(1,N),'b--','LineWidth',2);
set(gca,'FontSize',14)
legend(h1,'Benchmark','Hand-to-Mouth','Location','NorthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(e) Income, $y$','interpreter','latex')
xlabel('Years','FontSize',14,'interpreter','latex')

subplot(3,2,6)
h1 = plot(time_bench,CEQ_t_bench,'g',time,CEQ_t,'r',time,zeros(1,N),'b--','LineWidth',2);
set(gca,'FontSize',14)
legend(h1,'Benchmark','Hand-to-Mouth','Location','SouthEast')
xlim([0 Tmax])
set(findobj('color','g'),'Color',[0 0.5 0]);
title('(f) Worker Period Utility, $u(c,\ell)$','interpreter','latex')
xlabel('Years','FontSize',14,'interpreter','latex')

print -depsc graphs/appendix/fig2_deviations_comparison_htm.eps
% print -dpdf ../graphs/fig2_deviations_comparison_htm.pdf
