clear all; clc; close all;

Theta1 = 1.1;
Theta2 = 1;
p1 = 1;
p2 = 1;
al = 1/3;
eta = 1.05;
ga = (al/eta)/(1-al + al/eta);

d = 0.15;
rho = 0.03;

tau_bar = ga/(ga + (1-ga)*(d/rho));

L = .5;

x0 = 1;
x10 = x0;
x20 = x0;

lx1_ratio = ((d - rho)/(p1*Theta1*al/eta))^(1/(1-ga));
lx2_ratio = ((d - rho)/(p2*Theta2*al/eta))^(1/(1-ga));


T = 150;
N = 10000;
dt = T/N;
time = dt*(1:N);
% time1 = time(1:N-1);

T1 = 90;
N1 = round(T1/dt);

maxit1 = 30;
maxit2 = 30;
crit = 10^(-6);

ep = 10^(-6);

%SHOOTING ALGORITHM FINDING tau10 and tau20
x1_t(1) = x10;
x2_t(1) = x20;

tau10 = -.29291;
tau20 = 0.1258;

tau1_max = tau_bar;
tau1_min = -0.6;


for i=1:maxit1
tau1_t(1) = tau10;

tau20 = 0.1;
tau2_max = 0.7;
tau2_min = -0.1;

for j=1:maxit2
tau2_t(1) = tau20;
for n=1:N
    l1_t(n) = (max(1-tau1_t(n),ep)*p1*Theta1)^(1/ga)*x1_t(n)/((max(1-tau1_t(n),ep)*p1*Theta1)^(1/ga)*x1_t(n) + (max(1-tau2_t(n),ep)*p2*Theta2)^(1/ga)*x2_t(n))*L;
    l2_t(n) = (max(1-tau2_t(n),ep)*p2*Theta2)^(1/ga)*x2_t(n)/((max(1-tau1_t(n),ep)*p1*Theta1)^(1/ga)*x1_t(n) + (max(1-tau2_t(n),ep)*p2*Theta2)^(1/ga)*x2_t(n))*L;
    l2x2_ratio(n) = (max(1-tau2_t(n),ep)*p2*Theta2)^(1/ga)/((max(1-tau1_t(n),ep)*p1*Theta1)^(1/ga)*x1_t(n) + (max(1-tau2_t(n),ep)*p2*Theta2)^(1/ga)*x2_t(n))*L;
    xxx1 = ga*(1-tau1_t(n))*al/eta*p1*Theta1*(l1_t(n)/x1_t(n))^(1-ga);
    xxx2 = ga*(1-tau2_t(n))*al/eta*p2*Theta2*l2x2_ratio(n)^(1-ga);
    tau1_t(n+1) = dt*(d*(tau1_t(n)-ga) + xxx1) + tau1_t(n);
    tau2_t(n+1) = dt*(d*(tau2_t(n)-ga) + xxx2) + tau2_t(n);
    x1_t(n+1)  = dt*(al/eta*p1*Theta1*x1_t(n)^ga.*l1_t(n)^(1-ga) + (rho-d)*x1_t(n)) + x1_t(n);
    x2_t(n+1)  = dt*(al/eta*p2*Theta2*l2x2_ratio(n)^(1-ga)*x2_t(n) + (rho-d)*x2_t(n)) + x2_t(n);
end

converge2(j) = tau2_t(N1)-tau2_t(N1-1);
tau2_min_j(j) = tau2_min;
tau2_max_j(j) = tau2_max;
tau20_j(j) = tau20;

%UPDATE tau20
if converge2(j)>crit
    tau2_max = tau20;
    tau20 = 0.5*(tau20+tau2_min);
elseif converge2(j)<-crit;
    tau2_min = tau20;
    tau20 = 0.5*(tau20+tau2_max);
elseif abs(converge2(j))<crit;
    display('Shooting algorithm converged, tau20 =')
    disp(tau20)
    break
end

if j==maxit2
    disp('CAREFUL: SHOOTING ALGORITHM DID NOT CONVERGE')
    disp('i iteration = ')
    disp(i)
end

end

converge1(i) = tau1_t(N1)-tau1_t(N1-1);
tau1_min_j(i) = tau1_min;
tau1_max_j(i) = tau1_max;
tau10_j(i) = tau10;

%UPDATE tau10
if converge1(i)>crit
    tau1_max = tau10;
    tau10 = 0.5*(tau10+tau1_min);
elseif converge1(i)<-crit;
    tau1_min = tau10;
    tau10 = 0.5*(tau10+tau1_max);
elseif abs(converge1(i))<crit;
    display('Shooting algorithm converged, tau10 =')
    disp(tau10)
    break
end

end
x1_t(N+1)=[]; x2_t(N+1)=[]; tau1_t(N+1)=[]; tau2_t(N+1)=[];


plot([tau20_j;tau2_max_j;tau2_min_j]')

plot(time,x1_t,time,x2_t)
plot(time,l1_t,time,l2_t)

plot(time,tau1_t,time,tau2_t,time,tau_bar*ones(1,N))
legend('tau1','tau2')
xlim([0 T1])


x1CE_t(1) = x10;
x2CE_t(1) = x20;
for n=1:N
    l1CE_t(n) = (p1*Theta1)^(1/ga)*x1CE_t(n)/((p1*Theta1)^(1/ga)*x1CE_t(n) + (p2*Theta2)^(1/ga)*x2CE_t(n))*L;
    l2CE_t(n) = (p2*Theta2)^(1/ga)*x2CE_t(n)/((p1*Theta1)^(1/ga)*x1CE_t(n) + (p2*Theta2)^(1/ga)*x2CE_t(n))*L;
    y1CE_t(n) = Theta1*x1CE_t(n)^ga.*l1CE_t(n)^(1-ga);
    y2CE_t(n) = Theta2*x2CE_t(n)^ga.*l2CE_t(n)^(1-ga);
    l2x2CE_ratio(n) = (p2*Theta2)^(1/ga)/((p1*Theta1)^(1/ga)*x1CE_t(n) + (p2*Theta2)^(1/ga)*x2CE_t(n))*L;
    x1CE_t(n+1)  = dt*(al/eta*p1*y1CE_t(n) + (rho-d)*x1CE_t(n)) + x1CE_t(n);
    x2CE_t(n+1)  = dt*(al/eta*p2*Theta2*l2x2CE_ratio(n)^(1-ga)*x2CE_t(n) + (rho-d)*x2CE_t(n)) + x2CE_t(n);
end
x1CE_t(N+1)=[]; x2CE_t(N+1)=[];

% plot(time,tau1_t,time,tau2_t)
% legend('tau1','tau2')

subsidy = (1-tau1_t)./(1-tau2_t)-1;

figure(1)
set(gcf,'PaperPosition',[0 0 15 5])
subplot(1,2,1)
plot(time,subsidy,'b','LineWidth',2)
set(gca,'FontSize',16)
xlim([0 T1])
xticks([0:20:80])
ylim([0.15 0.25])
yticks([0.16:.02:.24])
title('(a) Subsidy to Sector 0','interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')

subplot(1,2,2)
plot(time,x1CE_t,'r--',time,x1_t,'r',time,x2CE_t,'b--',time,x2_t,'b','LineWidth',2)
set(gca,'FontSize',16)
% legend('Prod. Sector, Equilibrium','Prod. Sector, Planner','Unprod. Sector, Equilibrium','Unprod. Sector, Planner','Location','East')
title('(b) Sectoral Wealth, $x_i$','interpreter','latex')
text(40,1.75,'Sector 0','FontSize',16,'interpreter','latex')
text(25,.75,'Sector 1','FontSize',16,'interpreter','latex')
xlabel('Years','FontSize',16,'interpreter','latex')
xlim([0 T1])
xticks([0:20:80])

print -depsc graphs/two_sector.eps
